// -*- C++ -*-
/*!
 * @file  ObjectPoseEstimation.h
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

#ifndef OBJECTPOSEESTIMATION_H
#define OBJECTPOSEESTIMATION_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "ObjectPoseEstimation2ServiceSVC_impl.h"
#include "VisionSVC_impl.h"
#include "SRSensorDataStub.h"
// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "ModelDefinitionCommonServiceStub.h"
#include "EstimationResultViewerServiceStub.h"

// </rtc-template>

#include <string>
#include "ip.h"

#include "executeOPE.h"

using namespace RTC;
using namespace SensorRTC;

/*!
 * @class ObjectPoseEstimation
 * @brief ModuleDescription
 *
 */
class ObjectPoseEstimation
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  ObjectPoseEstimation(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~ObjectPoseEstimation();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
	 float m_DistanceThresh;
	 float m_ConfidenceThresh;
 
	 float m_Chess_size;
     int   m_Pat_row;
	 int   m_Pat_col;

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
	 SensorRTC::SRSensor::idl::TimedPoint3DList m_3DPointCloud;
  /*!
   */
  InPort<SensorRTC::SRSensor::idl::TimedPoint3DList> m_3DPointCloudIn;
  SensorRTC::SRSensor::idl::TimedSwissRanger m_SwissRangerData;
  /*!
   */
  InPort<SensorRTC::SRSensor::idl::TimedSwissRanger> m_SwissRangerDataIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedDoubleSeq m_RecognitionResult;
  /*!
   */
  OutPort<TimedDoubleSeq> m_RecognitionResultOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   */
  RTC::CorbaPort m_MDServicePort;
  /*!
   */
  RTC::CorbaPort m_OPEServicePort;
  /*!
   */
  RTC::CorbaPort m_ERVServicePort;
  /*!
   */
  RTC::CorbaPort m_RecognitionServicePort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   */
  OPEServiceSVC_impl m_OPEService;
  /*!
   */
  RecognitionServiceSVC_impl m_RecognitionService;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> m_MDService;
  /*!
   */
  RTC::CorbaConsumer<ObjectPoseEstimationRTC::idl::ERVService> m_ERVService;
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
	
	 CRITICAL_SECTION gCritSec;
	 SRSensor::idl::TimedSwissRanger *pSwissRangerData;
	 OPEServiceSVC_impl::Point3DList *pPoint3DData;

	char key;

	//肷郂f
	ModelDefinitionRTC::idl::ObjectModel EstimationModel;
	Model EstimationModel_use;
	std::vector<Model> EstimationModel_use_List;

	//3_Q
	CvPoint3D32f PointCloud[25344];

	//ۂɕ\邽߂̉摜
	IplImage *dstObjectImg;

	//]s
	std::vector<double> TempCRecogResult;
	std::vector<double> CommonRecogResult;

	//茋
	RTC::Pose3D EstimateResult;
	std::vector<EstimateParameter> EstimateResultList;

	//C[t[`p
	std::vector<CvPoint3D32f> ResultVertex;
	std::vector<std::vector<CvPoint3D32f>> ResultVertexList;
	ObjectPoseEstimationRTC::idl::ERVService::Point3DList points;

  // </rtc-template>

  // <rtc-template block="private_operation">
	void calcMemberofR(const RTC::Pose3D pose, std::vector<double>& RecogResult);

	void DrawDistanceImg( IplImage **distanceimg);
  // </rtc-template>

	ModelDefinitionRTC::idl::ModelIDList *IDList;

};


extern "C"
{
  DLL_EXPORT void ObjectPoseEstimationInit(RTC::Manager* manager);
};

#endif // OBJECTPOSEESTIMATION_H
