/**********************************************************************/
/*                                                                    */
/*                    Header File for Utilities                       */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/*      Copyright(c) 1990-2010    Jun MIURA
                     Department of Computer Science and Engineering,
		     Toyohashi University of Technology
		     Toyohashi, Aichi 441-8580, JAPAN
                     (phone) +81-532-44-6773 (facsimile) +81-532-44-6757
                     (E-mail) jun.miura@tut.jp                   */

#ifndef MYUTIL_H
#define MYUTIL_H

/*
 *  several useful values
 */
/* 
 * typedef int	   BOOLEAN;
 */

#define TRUE	1
#define FALSE	0

#define bool(x)	(((x) == 1) ? "TRUE " : "FALSE")
#define bool_symbol(x)	(((x) == 1) ? "O" : "X")

#define STDIN	0
#define STDOUT	1

#define MAXCHARS	256

#define VERY_LARGE	9999999999999.9
#define VERY_SMALL	-999999999999.9
#define VERY_LARGE_INT	999999
#define VERY_SMALL_INT	-999999

#define MILLION		1000000.0

/*
 *  useful macros
 */
#define	sqr(x)		((x)*(x))
#define max(x, y)	((x > y) ? x : y)
#define min(x, y)	((x < y) ? x : y)
#define deg_to_rad(x)	((x) * M_PI / 180.0)
#define rad_to_deg(x)	((x) * 180.0 / M_PI)
#define odd(x)		((((x) % 2) == 1) ? TRUE : FALSE)
#define even(x)		((((x) % 2) == 0) ? TRUE : FALSE)
#define debug_x(v)	fprintf(stderr, #v"=0x%08x\n", (v))
#define debug_i(v)	fprintf(stderr, #v"=%d\n", (v))
#define debug_d(v)	fprintf(stderr, #v"=%f\n", (v))
#define in_range(x, mi, ma)	((x < mi) ? FALSE : ((x > ma) ? FALSE : TRUE))
#define ipow(x, y)	(int)pow((double)(x), (double)(y))
#define ihypot(x, y)	(int)hypot((double)(x), (double)(y))
#define sign(x)		((x >= 0.0) ? 1.0 : -1.0)

/*
 *  make a string of suffix for an ordinal number
 */
#define OrdinalSuffix(x)	(((((x) % 100) == 11) ||\
				  (((x) % 100) == 12) ||\
				  (((x) % 100) == 13)) ? "th" :\
				 ((((x) % 10) == 1) ? "st" :\
				  ((((x) % 10) == 2) ? "nd" :\
				   ((((x) % 10) == 3) ? "rd" : "th"))))

#define New(x)			(x *)malloc(sizeof(x))
#define CNew(x, num)		(x *)calloc(num, sizeof(x))
#define BNew(x, row, col)	(x **)balloc(row, col, sizeof(x))

#define BOld(x)			bfree(x)
#define COld(x)			free(x)
#define Old(x)			free(x)

/*
 *  next and before for the cyclic object.
 */
#define Next(x, mmax)		((x < (mmax - 1)) ? (x + 1) : 0)
#define Before(x, mmax)		((x == 0) ? (mmax - 1) : (x - 1))

#endif /*  MYUTIL_H  */
