#pragma once


#include "cvutil.h"


namespace opencv
{
namespace cvutil
{

	class	Undistortor
	{
	private:
		/* cݕ␳WLUT̗̈m*/
		cvutil::Image<IPL_DEPTH_32S> map;
		cvutil::Image<IPL_DEPTH_32F> mixture;

	public:
		Undistortor()
		{}

		~Undistortor()
		{}

	public:
		void initialize(
			double width, double height,
			double u0, double v0, double fu, double fv,
			double k1, double k2, double p1, double p2 )
		{
			int w = (int) width, h = (int) height;
			
			map.create( w, h, 2 );			map.clear();
			mixture.create( w, h, 4 );	mixture.clear();
	
			#ifdef CVUTIL_UNDISTORTOR_SHIFT_CENTER
			int u_u0 = int( 0.5*width - u0 + 0.5 );
			int v_v0 = int( 0.5*height - v0 + 0.5 );
			#endif
				
			int u, v;
			double _fx = 1.f/fu, _fy = 1.f/fv;
			
			/* We[ȕ */
			for( v = 0; v < h; ++v ) {
				double y = (v - v0)*_fy;
				double y2 = y*y;
				double ky = 1 + (k1 + k2*y2)*y2;
				double k2y = 2*k2*y2;
				double _2p1y = 2*p1*y;
				double _3p1y2 = 3*p1*y2;
				double p2y2 = p2*y2;

				for( u = 0; u < w; ++u ) {
					double x = (u - u0)*_fx;
					double x2 = x*x;
					double kx = (k1 + k2*x2)*x2;
					double d = kx + ky + k2y*x2;
					double _u = fu*(x*(d + _2p1y) + p2y2 + (3*p2)*x2) + u0;
					double _v = fv*(y*(d + (2*p2)*x) + _3p1y2 + p1*x2) + v0;
			
					#ifdef CVUTIL_UNDISTORTOR_SHIFT_CENTER
					 _u -= u_u0;
					 _v -= v_v0;
					#endif
					 
					 //printf("(%d, %d) -> (%.1f, %.1f)\n", u, v, _u, _v);
					 //int idx = u + w * v;
					 /* ϊ̍W̃`FbN */
					 //if( 0 <= _u && _u < w && 0 <= _v && _v < h ) {
					 if( map.isIn( int(_u), int(_v) ) ) {
						 /* ČvZQƊWۑ(J[摜Ȃ̂ ~ch_n) */
						 //map(u,v) = ch_n * ((int)_u + w * (int)_v);
						 map(u,v, 0) = (int)_u;
						 map(u,v, 1) = (int)_v;

						 /* o */
						 _u -= (int)_u;	_v -= (int)_v;
						 
						 /* oCjAԌW̕ۑ */
						 mixture(u,v,0) = float((1.0 - _u) * (1.0 - _v));
						 mixture(u,v,1) = float(_u * (1.0 - _v));
						 mixture(u,v,2) = float((1.0 - _u) * _v);
						 mixture(u,v,3) = float(_u * _v);
/*						 mixture(u,v,0) = 1.0;
						 mixture(u,v,1) = 0.0;
						 mixture(u,v,2) = 0.0;
						 mixture(u,v,3) = 0.0;*/
					}
					else{
						mixture(u,v,0) = mixture(u,v,1) = mixture(u,v,2) = mixture(u,v,3) = 0;
					}
				}
			}
		}

		/**
			J[摜ɘcݕ␳
			@	Image *src	:	͉摜
			@	Image *dst	:	cݕ␳̏o͉摜
		*/
		template< int ELEMENT_T >
		void process( const cvutil::Image< ELEMENT_T > &src, cvutil::Image< ELEMENT_T > &dst )
		{
			const int NW = 0, NE = src.getNChannels(), SW = NW + src.getWidthStep(), SE = NE + src.getWidthStep();
			const cvutil::Image<IPL_DEPTH_32F> &mix = mixture;
			int _u, _v;
//#pragma warning( push )
//#pragma warning( disable: 4244 )
			for( int v = 0; v < dst.getHeight(); ++v ) {
				for( int u = 0; u < dst.getWidth(); ++u ) {
					for( int ch = 0; ch < dst.getNChannels(); ++ ch ) {
						/*
						dst(u,v,ch)
							= (cvutil::Image< ELEMENT_TYPE >::ImageElemType)
							( *(&src(0,0) + map(u,v) + NW + ch) * mix(u,v,0) + *(&src(0,0) + map(u,v) + NE + ch) * mix(u,v,1)
							+ *(&src(0,0) + map(u,v) + SW + ch) * mix(u,v,2) + *(&src(0,0) + map(u,v) + SE + ch) * mix(u,v,3));
						*/
						_u = map(u,v,0);
						_v = map(u,v,1);
						dst(u,v,ch)
							= Pixel::CvImageElemTypeTraits< ELEMENT_T >::ImageElemType
							( src(_u+0,_v+0,ch)*mix(u,v,0) + src(_u+1,_v+0,ch)*mix(u,v,1)
							+ src(_u+0,_v+1,ch)*mix(u,v,2) + src(_u+1,_v+1,ch)*mix(u,v,3));
					}
				}
			}
//#pragma warning( pop )
		}
	};
	
}
}